#ifndef USER_SENSOR_INFO_H__
#define USER_SENSOR_INFO_H__
#include <stdbool.h>
#include <stdint.h>

#define SYNC_ADDR_LEN 5 // don't change

typedef enum
{
    SYNC_TYPE_SLAVE,
    SYNC_TYPE_MASTER,
} synchronization_mode_t;

typedef enum
{
    SYNC_STATE_OFF,
    SYNC_STATE_ON,
} synchronization_state_t;

typedef struct data_message
{
    uint8_t buffer[50];
    uint8_t length;
    uint8_t secondary_buffer[50];
    uint8_t secondary_length;
    uint64_t second;
    uint32_t sample_count;
} data_message_t;

typedef enum
{ // NOTE: DO NOT modifiy!!!!!!!!!!!!!
  // NOTE: DO NOT modifiy!!!!!!!!!!!!!
    KFORCE_UNKNOWN_DEVICE = 0,
    KFORCE_PLATE_L = 0x12,
    KFORCE_PLATE_R = 0x13,
    KFORCE_LINK = 0x15,
    KFORCE_GRIP = 0x10,
    KFORCE_MUSCLE = 0x11,
    KFORCE_BUBBLE = 0x14,
    KFORCE_SENS = 0x16,
    KFORCE_HEXA_L = 0x17,
    KFORCE_HEXA_R = 0x18,
    KFORCE_LINK_SENS = 0x19,
    KFORCE_MUSCLE_SENS = 0x1A,
    KFORCE_GRIP_SENS = 0x1B,
    KFORCE_BUBBLE_SENS = 0x1C,
    KFORCE_EMG = 0x1D,
    KFORCE_KPOWER = 0x1E,

    KFORCE_DELTA3_L = 0x23,
    KFORCE_DELTA3_R = 0x24,
} sensor_type_t;

// NOTE: DO NOT modify the struct add a new one
typedef struct
{
    sensor_type_t sensor_type;
    uint64_t sn;
    uint8_t hw_version[20];
    uint8_t addr[6];
    uint16_t max_battery_voltage;
    uint16_t max_charge_current;
    uint16_t battery_voltage_rate;
} sensor_persistent_info_basic_t;

// NOTE: DO NOT modify the struct add a new one
typedef struct
{
    bool is_yt_calibration_active;

    uint32_t adc_zero_ch[4];

    uint32_t baseline_ch0;
    uint32_t baseline_ch1;
    uint32_t baseline_ch2;
    uint32_t baseline_ch3;

    uint32_t baseline_24bits_ch0;
    uint32_t baseline_24bits_ch1;
    uint32_t baseline_24bits_ch2;
    uint32_t baseline_24bits_ch3;

    uint32_t coef_I;
    uint32_t coef_II;
    uint32_t coef_III;
    uint32_t coef_IV;
} sensor_persistent_info_force_t;

// NOTE: DO NOT modify the struct add a new one
typedef struct
{
    uint32_t adc_zero_ch[12];

    uint32_t baseline_ch4;
    uint32_t baseline_ch5;
    uint32_t baseline_ch6;
    uint32_t baseline_ch7;
    uint32_t baseline_ch8;
    uint32_t baseline_ch9;
    uint32_t baseline_ch10;
    uint32_t baseline_ch11;
    uint32_t baseline_ch12;
    uint32_t baseline_ch13;
    uint32_t baseline_ch14;
    uint32_t baseline_ch15;

    uint32_t baseline_24bits_ch4;
    uint32_t baseline_24bits_ch5;
    uint32_t baseline_24bits_ch6;
    uint32_t baseline_24bits_ch7;
    uint32_t baseline_24bits_ch8;
    uint32_t baseline_24bits_ch9;
    uint32_t baseline_24bits_ch10;
    uint32_t baseline_24bits_ch11;
    uint32_t baseline_24bits_ch12;
    uint32_t baseline_24bits_ch13;
    uint32_t baseline_24bits_ch14;
    uint32_t baseline_24bits_ch15;
} sensor_persistent_info_force_extra_t;

// NOTE: DO NOT modify the struct add a new one
typedef struct
{
    uint16_t measurement_rate_low;
    uint16_t measurement_rate_high;
} sensor_persistent_info_v2_t;

// NOTE: DO NOT modify the struct add a new one
typedef struct
{
    uint8_t auto_sleep_time;
    uint8_t auto_sleep_enable;
    uint8_t auto_wakeup_time;
    uint8_t auto_wakeup_time_enable;
    uint8_t auto_wakeup_force[4];
    uint8_t auto_wakeup_force_enable;
    uint32_t ble_adv_timeout;
    uint32_t auto_wakeup_usb_enable;
} sensor_persistent_info_lowpower_t;

// NOTE: DO NOT modify the struct add a new one
typedef struct
{
    uint32_t calib_coef_acc_len;
    uint8_t calib_coef_acc[1024];
    uint32_t calib_coef_gyro_len;
    uint8_t calib_coef_gyro[1024];
    uint32_t calib_coef_compass_len;
    uint8_t calib_coef_compass[1024];
    bool is_campass_enabled;
} sensor_persistent_info_imu_t;

// NOTE: DO NOT modify the struct add a new one
typedef struct
{
    uint8_t max_conn_interval;
    uint8_t min_conn_interval;
    uint8_t normal_gap_evt_length;
    uint8_t high_speed_gap_evt_length;
    uint8_t is_high_speed; // 0xff is high speed 0x00 is normal
} sensor_persistent_ble_t;

// NOTE: DO NOT modify the struct add a new one
typedef struct
{
    uint16_t size;
    uint16_t acc_range;
    uint16_t gyro_range;
} sensor_persistent_info_imu_cfg_t;

// NOTE: DO NOT modify the struct add a new one
typedef struct
{
    uint16_t size;
    uint8_t connected_pwr;
    uint8_t adv_pwr;
    uint8_t sync_pwr;
} sensor_persistent_info_tx_pwr_t;

typedef struct
{
    // sensor_retention_info_t *p_retention_info;
    sensor_persistent_info_basic_t __attribute__((aligned(4))) persistent_info_basic;
    sensor_persistent_info_force_t __attribute__((aligned(4))) persistent_info_force;
    sensor_persistent_info_v2_t __attribute__((aligned(4))) persistent_info_v2;
    sensor_persistent_info_lowpower_t __attribute__((aligned(4))) persistent_info_lowpower;
    sensor_persistent_info_imu_t __attribute__((aligned(4))) persistent_info_imu;
    sensor_persistent_info_imu_cfg_t __attribute__((aligned(4))) persistent_info_imu_cfg;
    sensor_persistent_ble_t __attribute__((aligned(4))) persistent_info_ble;
    sensor_persistent_info_tx_pwr_t __attribute__((aligned(4))) persistent_info_tx_pwr;
    uint8_t __attribute__((aligned(4))) memory_table_v2[256]; // NOTE: DO NOT modify the size of matrix NOTE: Create new matrix
    uint8_t __attribute__((aligned(4)))
    memory_table_addr_10[256]; // NOTE: DO NOT modify the size of matrix NOTE: Create new matrix

    synchronization_mode_t sync_mode;
    volatile synchronization_state_t sync_state;

    bool pending_synchronization; // clock diff is ready to be apply
    bool init_synchronization;    // during first synchronisation, before measurement is started, de
                                  // time diff could be larger.
    uint8_t sync_addr[SYNC_ADDR_LEN];
    uint8_t sync_ch;

    volatile uint16_t nb_raw_sample;
    volatile uint32_t nb_totale;

    volatile float battery_voltage;
    volatile float battery_current;
    volatile uint8_t battery_percentage;

} sensor_info_t;
#endif // USER_SENSOR_INFO_H__